import _extends from "@babel/runtime/helpers/esm/extends";
import { cloneElement, Fragment, useCallback, useEffect, useRef, useState } from 'react';
import cx from 'classnames';
import PropTypes from 'prop-types';
import usePortal from '@s-ui/react-hook-use-portal';
import { BASE_CLASS, getPlacement, OVERLAY_TYPES, PLACEMENTS, SHAPES, SIZES } from './config.js';
import RenderActions from './RenderActions.js';
import SelectIcon from './components/SelectIcon.js';
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
function usePrevious(value) {
  var ref = useRef();
  useEffect(function () {
    ref.current = value;
  }, [value]);
  return ref.current;
}
var popoverBaseClass = BASE_CLASS + "-popover";
var MoleculeSelectPopover = function MoleculeSelectPopover(_ref) {
  var acceptButtonText = _ref.acceptButtonText,
    acceptButtonOptions = _ref.acceptButtonOptions,
    cancelButtonText = _ref.cancelButtonText,
    cancelButtonOptions = _ref.cancelButtonOptions,
    customButtonText = _ref.customButtonText,
    customButtonOptions = _ref.customButtonOptions,
    children = _ref.children,
    _ref$forceClosePopove = _ref.forceClosePopover,
    forceClosePopover = _ref$forceClosePopove === void 0 ? false : _ref$forceClosePopove,
    fullWidth = _ref.fullWidth,
    hideActions = _ref.hideActions,
    IconArrowDown = _ref.iconArrowDown,
    id = _ref.id,
    _ref$isDisabled = _ref.isDisabled,
    isDisabled = _ref$isDisabled === void 0 ? false : _ref$isDisabled,
    _ref$isSelected = _ref.isSelected,
    isSelected = _ref$isSelected === void 0 ? false : _ref$isSelected,
    _ref$onAccept = _ref.onAccept,
    onAccept = _ref$onAccept === void 0 ? function () {} : _ref$onAccept,
    _ref$onCancel = _ref.onCancel,
    onCancel = _ref$onCancel === void 0 ? function () {} : _ref$onCancel,
    _ref$onCancelButtonCl = _ref.onCancelButtonClick,
    onCancelButtonClick = _ref$onCancelButtonCl === void 0 ? function () {} : _ref$onCancelButtonCl,
    _ref$onClose = _ref.onClose,
    onClose = _ref$onClose === void 0 ? function () {} : _ref$onClose,
    _ref$onCustomAction = _ref.onCustomAction,
    onCustomAction = _ref$onCustomAction === void 0 ? function () {} : _ref$onCustomAction,
    _ref$onOpen = _ref.onOpen,
    onOpen = _ref$onOpen === void 0 ? function () {} : _ref$onOpen,
    _ref$overlayContentRe = _ref.overlayContentRef,
    overlayContentRef = _ref$overlayContentRe === void 0 ? {} : _ref$overlayContentRe,
    _ref$overlayType = _ref.overlayType,
    overlayType = _ref$overlayType === void 0 ? OVERLAY_TYPES.NONE : _ref$overlayType,
    placement = _ref.placement,
    _ref$removeButtonOpti = _ref.removeButtonOptions,
    removeButtonOptions = _ref$removeButtonOpti === void 0 ? null : _ref$removeButtonOpti,
    renderContentWrapperProp = _ref.renderContentWrapper,
    renderSelectProp = _ref.renderSelect,
    renderActionsProp = _ref.renderActions,
    selectText = _ref.selectText,
    shape = _ref.shape,
    _ref$size = _ref.size,
    size = _ref$size === void 0 ? 'm' : _ref$size,
    title = _ref.title;
  var _useState = useState(false),
    isOpen = _useState[0],
    setIsOpen = _useState[1];
  var _useState2 = useState(cx("" + popoverBaseClass, popoverBaseClass + "--" + getPlacement(placement))),
    popoverClassName = _useState2[0],
    setPopoverClassName = _useState2[1];
  var previousIsOpen = usePrevious(isOpen);
  var selectRef = useRef();
  var contentWrapperRef = useRef();
  var _usePortal = usePortal({
      target: overlayContentRef.current
    }),
    Portal = _usePortal.Portal;
  var hasOverlay = Boolean(overlayContentRef.current) && overlayType !== OVERLAY_TYPES.NONE;
  useEffect(function () {
    forceClosePopover && setIsOpen(false);
  }, [forceClosePopover]);
  useEffect(function () {
    /**
     * Only run open events:
     *  - After first render
     *  - When isOpen actually changes
     *  - When placement changes
     **/
    if (typeof previousIsOpen === 'undefined' || isOpen === previousIsOpen) {
      return;
    }
    var getPopoverClassName = function getPopoverClassName() {
      if (isOpen && [PLACEMENTS.AUTO_END, PLACEMENTS.AUTO_START].includes(placement)) {
        var _contentWrapperRef$cu;
        var _ref2 = ((_contentWrapperRef$cu = contentWrapperRef.current) == null ? void 0 : _contentWrapperRef$cu.getBoundingClientRect()) || {},
          left = _ref2.left,
          right = _ref2.right;
        var outFromTheLeftSide = left < 0;
        var outFromTheRightSide = right > (window.innerWidth || document.documentElement.clientWidth);
        if (outFromTheRightSide) {
          return cx("" + popoverBaseClass, popoverBaseClass + "--left");
        } else if (outFromTheLeftSide) {
          return cx("" + popoverBaseClass, popoverBaseClass + "--right");
        }
      }
      return cx("" + popoverBaseClass, popoverBaseClass + "--" + getPlacement(placement));
    };
    setPopoverClassName(getPopoverClassName());
    var openEvent = isOpen ? onOpen : onClose;
    openEvent();
  }, [isOpen, onClose, onOpen, previousIsOpen, placement]);
  var handleOnAccept = function handleOnAccept() {
    setIsOpen(false);
    onAccept();
  };
  var handleOnCustomAction = function handleOnCustomAction() {
    onCustomAction();
  };
  var handleCancelButtonClick = function handleCancelButtonClick() {
    onCancelButtonClick();
    handleOnCancel();
  };
  var handleOnCancel = useCallback(function () {
    setIsOpen(false);
    onCancel();
  }, [onCancel]);
  useEffect(function () {
    var handleClickOutside = function handleClickOutside(e) {
      if (isOpen && contentWrapperRef.current && !contentWrapperRef.current.contains(e.target) && !selectRef.current.contains(e.target)) {
        handleOnCancel();
      }
    };
    window.addEventListener('mousedown', handleClickOutside);
    return function () {
      window.removeEventListener('mousedown', handleClickOutside);
    };
  }, [handleOnCancel, isOpen]);
  var handleOpenToggle = function handleOpenToggle() {
    if (isOpen) {
      setIsOpen(false);
      handleOnCancel();
      return;
    }
    setIsOpen(true);
  };
  var renderProp = function renderProp(render, props) {
    return typeof render === 'function' ? render(props) : /*#__PURE__*/cloneElement(render, _extends({}, render.props, props));
  };
  var renderSelect = function renderSelect() {
    var _cx, _cx2;
    var newSelectProps = _extends({
      ref: selectRef,
      className: cx(BASE_CLASS + "-select", shape && BASE_CLASS + "-select--" + shape, BASE_CLASS + "-select--" + size, (_cx = {}, _cx[BASE_CLASS + "-select--withRemoveOption"] = removeButtonOptions, _cx['is-open'] = isOpen, _cx['is-selected'] = isSelected, _cx)),
      onClick: handleOpenToggle
    }, id && {
      id: id
    });
    if (renderSelectProp) {
      var _renderSelectProp$pro;
      var newRenderSelectProps = _extends({}, newSelectProps, {
        isOpen: isOpen,
        onClick: (_renderSelectProp$pro = renderSelectProp.props) != null && _renderSelectProp$pro.onClick ? function (ev) {
          renderSelectProp.props.onClick(ev);
          handleOpenToggle(ev);
        } : handleOpenToggle
      });
      return renderProp(renderSelectProp, newRenderSelectProps);
    }
    return /*#__PURE__*/_jsxs("div", _extends({
      className: cx(BASE_CLASS + "-select", shape && BASE_CLASS + "-select--" + shape, BASE_CLASS + "-select--" + size, (_cx2 = {}, _cx2[BASE_CLASS + "-select--withRemoveOption"] = removeButtonOptions, _cx2['is-open'] = isOpen, _cx2['is-selected'] = isSelected, _cx2))
    }, newSelectProps, {
      children: [/*#__PURE__*/_jsx("span", {
        className: BASE_CLASS + "-selectText",
        children: selectText
      }), /*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-selectIcon",
        children: /*#__PURE__*/_jsx(SelectIcon, {
          iconArrowDown: IconArrowDown,
          removeButtonOptions: removeButtonOptions
        })
      })]
    }));
  };
  var renderContentWrapper = function renderContentWrapper() {
    var pieces = {
      actions: /*#__PURE__*/_jsx(RenderActions, {
        onCustomAction: handleOnCustomAction,
        customButtonText: customButtonText,
        customButtonOptions: customButtonOptions,
        onCancel: handleCancelButtonClick,
        cancelButtonText: cancelButtonText,
        cancelButtonOptions: cancelButtonOptions,
        onAccept: handleOnAccept,
        acceptButtonOptions: acceptButtonOptions,
        acceptButtonText: acceptButtonText,
        children: renderActionsProp
      }),
      content: children,
      contentWrapperRef: contentWrapperRef,
      // required for `handleClickOutside` to work
      isOpen: isOpen,
      setIsOpen: setIsOpen
    };

    /**
     * Custom content wrapper from render prop
     */
    if (renderContentWrapperProp) {
      return renderProp(renderContentWrapperProp, pieces);
    }

    /**
     * Default content wrapper as a popover
     */
    return pieces.isOpen && /*#__PURE__*/_jsxs("div", {
      className: popoverClassName,
      ref: contentWrapperRef,
      children: [/*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-popoverContent",
        children: pieces.content
      }), !hideActions && pieces.actions]
    });
  };
  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [/*#__PURE__*/_jsxs("div", {
      className: cx(BASE_CLASS, fullWidth && BASE_CLASS + "--fullWidth", isDisabled && 'is-disabled', renderContentWrapperProp && BASE_CLASS + "--hasCustomWrapper"),
      title: title,
      children: [renderSelect(), renderContentWrapper()]
    }), hasOverlay && /*#__PURE__*/_jsx(Portal, {
      as: Fragment,
      isOpen: isOpen,
      children: /*#__PURE__*/_jsx("div", {
        className: cx(BASE_CLASS + "-overlay", BASE_CLASS + "-overlay--" + overlayType)
      })
    })]
  });
};
MoleculeSelectPopover.displayName = 'MoleculeSelectPopover';
export default MoleculeSelectPopover;
export { OVERLAY_TYPES as selectPopoverOverlayTypes, PLACEMENTS as selectPopoverPlacements, SHAPES as selectPopoverShapes, SIZES as selectPopoverSizes, RenderActions };